%% Doing Economics with the Computer
% *Lecture 5: SOLUTIONS* $\\$
% Harris Dellas $\\$
% Lucas Kyriacou

%% Matlab Application: Financial Evaluation 2
% In this lecture we will explain how to use MATLAB to solve interest
% compounding problems. Borrowed from 'A Guide to MATLAB for Beginners and Experienced Users,' 
% Brian R. Hunt, Ronald L. Lipsman, Jonathan M. Rosenberg. 

%% Compulsory Readings
% Up to now, you should have read chapters 1 - 6 in Attaway (2019). The book is provided on Ilias. 

%% Further Exercises
% Up to now, you should have solved the practice questions in chapters 1 - 6 in Attaway (2019). 
% Solutions to the practice questions in chapter 3 - 6 are available on Ilias.

%% Housekeeping
% When you write a script, you should include these commands at the start.

clear all % Deletes memory
close all % closes open windows (figures etc.)
clc       % clears command window 

%% General Set-up
% Consider an account that has M dollars in it and pays monthly interest J. Suppose
% that, beginning at a certain point, an amount S is deposited monthly and
% no withdrawals are made.

%% Definitions
% * M: initial account balance in USD
% * J: monthly interest rate in percent
% * S: monthly deposits in USD
% * T: total amount at end of  period in USD
% * n: the number n of months elapsed

%% Interest rate compounding
%% (a) 
% Assume first that S=0. Using the Mortgage Payments application as a model, 
% derive an equation relating J, M, the number n
% of months elapsed, and the total T in the account after n months. Assume
% that the interest is credited on the last day of the month and the total T is
% computed on the last day after the interest is credited. $\\\\$
% *Answer:* $\\$
% Consider the status of the bank account on the last day of each month. At the end of
% the first month, the account has $M + M * J = M(1 + J)$ dollars. Then at the end
% of the second month the account contains $[M(1 + J)](1 + J) = M(1 + J)^2$ dollars.
% Similarly, at the end of n months, the account will hold $M(1 + J)^n$ dollars. So our
% formula is $T = M(1 + J)^n$.

%% (b) 
% Now assume that M = 0, that S is deposited on the first day of the month,
% and that as before interest is credited on the last day of the month, and the
% total T is computed on the last day after the interest has been credited.
% Once again, using the mortgage application as a model, derive an equation
% relating J, S, the number n of months elapsed, and the total T in the
% account after n months. $\\\\$
% *Answer:* $\\$
% Now we take M = 0 and S dollars deposited monthly. At the end of the first month
% the account has $S + S * J = S(1 + J)$ dollars. The next day, S dollars are added to
% that sum, and then at the end of the second month the account contains 
% $[S(1 + J) + S](1 + J) = S[(1 + J)^2 + (1 + J)]$ dollars. 
% Similarly, at the end of n months, the
% account will hold $S[(1 + J)^{n} + (1 + J)^{(n-1)} + ... + (1 + J)]$ dollars. Summing the
% geometric series, the amount T in the account after n months equals
% $T = S[((1 + J)^{(n+1)} - (J + 1))/((1 + J) - 1)] = S[((1 + J)^{(n+1)} - 1)/J - 1]$.

%% (c) 
% By combining the last two models derive an equation relating all of M,
% S, J, n, and T, now of course assuming that there is an initial amount in
% the account (M) as well as a monthly deposit (S). $\\\\$
% *Answer:* $\\$
% By combining the two models it is clear that, in an account with an initial balance M
% and monthly deposits S, the amount of money T after n months is given by
% $T = M(1 + J)^n + S[((1 + J)^{(n+1)}-1)/J-1]$.

%% (d) 
% If the annual interest rate is 5%, and no monthly deposits are made, how
% many years does it take to double your initial stash of money? What if the
% annual interest rate is 10%? $\\\\$
% *Answer:* $\\$
% We are asked to solve the equation
% $(1 + J)^n = 2$
% with the values for the monthly interest rate: $J = 0.05/12$ and $J = 0.1/12$.

% J = 0.05/12
syms n
months = solve((1 + 0.05/12)^n == 2);
years = double(months)/12

% J = 0.1/12
syms n
months = solve((1 + 0.1/12)^n == 2);
years = double(months)/12

% If you double the interest rate, you roughly halve the time required to achieve the goal.

%% (e) 
% In this and the next part, there is no initial stash. Assume an annual interest
% rate of 8%. How much do you have to deposit monthly to be a millionaire
% in 35 years (a career)?

format bank
1000000/(((1 + 0.08/12)^(12*35 + 1) - 1)/(0.08/12) - 1)

% You need to deposit 433.06 every month.

%% (f) 
% If the interest rate remains as in (e) and you can afford to deposit only
% 300 each month, how long do you have to work to retire a millionaire?

syms n
T = 300*(((1 + 0.08/12)^(n + 1) - 1)/(0.08/12) - 1);
months = solve(T - 1000000);
years = double(months)/12

%% (g) 
% You hit the lottery and win 100,000. You have two choices: take the
% money, pay the taxes, and invest whats left; or receive 100,000/240
% monthly for 20 years, depositing whats left after taxes. Assume that a
% 100,000 windfall costs you 35,000 in federal and state taxes, but that
% the smaller monthly payoff causes only a 20% tax liability. In which way
% are you better off 20 years later? Assume a 5% annual interest rate here.
% $\\$
% Note that this is exactly the problem one faces going into pension in CH with 
% regard to the choice of what to do with the accumulated second layer
% funds.

%% g) First Option
% First, taking all the money at once and saving it, after 20 years the 65,000 left after taxes
% generate:

option1 = (100000-35000)*(1 + 0.05/12)^(12*20)

% The stash grows to about 176,322. 

%% g) Second Option
% The second option yields
S = 0.8*(100000/240);
option2 = S*(((1 + 0.05/12)^(12*20 + 1) - 1)/(0.05/12) - 1)

% You accumulate only 137,582 this way.

% Taking the lump sum up front is clearly the better strategy.

%% (h)
% Historically, banks have paid roughly 5%, while the stock market has
% tended to return 8% on average over a 10-year period. So parts (e) and
% (f) relate more to investing than to saving. But suppose that the market
% in a 5-year period returns 13%, 15%, -3%, 5% and 10% in five successive
% years, and then repeats the cycle. (Note that the [arithmetic] average
% is 8%, though a geometric mean would be more relevant here.) Assume
% that 50,000 is invested at the start of a 5-year market period. How much
% does it grow to in 5 years? Now recompute four more times, assuming
% that you enter the cycle at the beginning of the second year, the third year,
% etc. Which choice yields the best/worst results? Can you explain why?
% Compare the results with a fixed-rate account paying 8%. 
% Assume simple annual interest. Redo the five investment computations, assuming that
% 10,000 is invested at the start of each year. Again analyze the results.

%% h.i)
rates = [.13, .15, -.03, .05, .10, .13, .15, -.03, .05 0.1]; %added 0.1
for k = 0:4
    T = 50000;
    for j = 1:5
        T = T*(1 + rates(k + j));
    end
    disp([k + 1, T])
end

%% Note on the result
% The results are all the same, you wind up with 72,794 USD regardless of
% were you enter in the cycle because the product $\prod_{1 \leq j \leq 5}$(1+rates(j)) is independent of
% the order in which you place the factors. If you put the 50,000 in a bank
% account paying 8%, you get $50000 * (1 + 0.08)^5 = 73,466$
% which is better than the market. The market's volatility hurts 
% you compared with the bank's stability. But of course that assumes 
% you can find a bank that will pay 8%. 

%% h.ii)
% Now let's see what happens with no 
% stash, but an annual investment instead. The analysis is more subtle here. 
% Set S = 10,000. At the end of one year, the account contains $S(1 + r_1)$; 
% then at the end of the second year $[S(1 + r_1) + S](1 + r_2)$, where we have 
% written $r_j$ for rates(j). 
% So at the end of five years, the amount in the account will 
% be the product of S and the number 
% $\prod_{j \geq 1}\left(1+r_{j}\right)+\prod_{j \geq 2}\left(1+r_{j}\right)+\prod_{j \geq 3}\left(1+r_{j}\right)+\prod_{j \geq 4}\left(1+r_{j}\right)+\left(1+r_{5}\right)$
% If you enter at a different year in the business cycle the terms get cycled appropriately.
% So now we can compute 

for k = 0:4
    T = ones(1, 5);
    for j = 1:5
        TT = 1;
        for l = j:5
            TT = TT*(1 + rates(k + l));
        end
        T(j) = TT;
    end
    disp([k + 1, 10000*sum(T)])
end

%% Note on the result
% Not surprisingly, all the amounts are less than what one obtains by investing the original
% 50,000 all at once. But in this model it matters where you enter the business
% cycle. Its clearly best to start your investment program when a recession is in force
% and end in a boom. Incidentally, the bank model yields in this case 
% $10000*(((1 + 0.08)^6 - 1)/0.08 - 1)=63359.290$
% which is better than some investment models and worse than others.

%% Additional Exercises
%% (a) 
% Assume now that M=1000, S = 0. Assume again that the interest is
% credited on the last day of the month
% and the total T is computed on the last day after the interest is credited.
% Let the monthly interest rate be generated randomly, following a 
% uniform distribution on the
% interval (0.003, 0.005).

a=0.003;
b=0.005;
J = a + (b-a).*rand(1,1); % J: monthly interest rate

M = 1000;                 % M: initial account balance
S = 0;                    % S: monthly deposits
n = 60;

T = M*(1+J).^n;

fprintf('Total amount in the account: %.3f USD \n',T);

%% (b)
% Now assume that M = 0, S=50, that S is deposited on the first day of the month,
% and that, as before, interest is credited on the last day of the month, and the
% total T is computed on the last day after the interest has been credited.
% Compute the total T in the account after n=60 months.

%% (b) Method 1
M = 0; 
S = 50;
n = 60;

T = S* (1 + J)*(((1+J)^(n)-1)/J) % Standard Formula
fprintf('Total amount in the account: %.3f USD \n',T);

%% (b) Method 2
T = M*(1 + J)^n + S*(((1 + J)^(n+1) - 1)/J - 1) % Standard Formula
fprintf('Total amount in the account: %.3f USD \n',T);

%% (b) Method 3
% At the end of 1, 2, 3,.. period we have an amount: 
% $1: 50*(1+r)$ $\\$
% $2: 50*(1+r)^2+50*(1+r)$ $\\$
% $3: 50*(1+r)^3+50*(1+r)^2+50*(1+r); ...$ $\\$
% Thus after n periods we have  $\\$
% $50*(1+r)*[(1+r)^{n-1} +(1+r)^{n-2}+ ...+1]$.   $\\$
% The term inside [] is a geometric series that starts with 1 and grows at
% the rate of $1+r$. The formula above corresponds to the sum in $[...]$
% Alternatively, note that $\\$
% $T(2)=T(1)+(1+r)^2$; $\\$
% $T(3)= T(2)+(1+r)^3,..., T(n+1)=T(n)+S*(1+r)^n$ $\\$
% and use the familiar for loop routine where $T(1)=50*(1+r)$.

T(1)=S*(1+J);

for i = 2:n   
    T(i)= S*(1+J)^i + T(i-1);
end

fprintf('Total amount in the account: %.3f USD \n',T(end));

%% (c) 
% Let $M = 1000, S=50$ and compute the total amount T in the account after $n=60$ months.
% Note that at the end of period 1, 2, 3, .., T, .. we have an amount 
% $T(1)= 1000*(1+r)   +50*(1+r);\\$
% $T(2)= 1000*(1+r)^2 +50*(1+r)^2 + 50(1+r);\\$
% $T(3)= 1000*(1+r)^3 +50*(1+r)^3 + 50*(1+r)^2+ 50*(1+r);\\$
% $T(4)= 1000*(1+r)^4 +50*(1+r)^4 + 50*(1+r)^3 +50*(1+r)^2 + 50*(1+r);\\$
% $T(4)-T(3)=1000*(1+r)^4 +50*(1+r)^4-1000*(1+r)^3=1000*(1+r)^3(1+r-1)+50*(1+r)^4$ $\\$ 
% $\rightarrow T(4)=T(3)+1000*(1+r)^3 (1+r-1)+50*(1+r)^4;\\$
% $T(n) = 1000*(1+r)^n + 50*(1+r)^n +50*(1+r)^{n-1}+50*(1+r)^{n-2} +..+50*(1+r);\\$
% Hence: $\\$
% $T(n+1) =T(n)+ 1000*(1+r)^{(n-1)}*r +50*(1+r)^n$

a=0.003;
b=0.005;
J = a + (b-a).*rand(1,1); % J: monthly interest rate

M = 1000; 
S = 50;
n = 60;

%% Method 1
T = M*(1 + J)^n + S*(((1 + J)^(n+1) - 1)/J - 1); % Standard Formula
fprintf('Total amount in the account: %.3f USD \n',T);

%% Method 2
T(1)= 1000*(1+J) +50*(1+J);

for i=2:n
    T(i) = 1000*((1+J)^(i-1))*J +50*(1+J)^i + T(i-1);
end
fprintf('Total amount in the account: %.3f USD \n',T(end));